/** 
    bpm_defs.h

    Main definitions for libbpm as well as doxygen intro documentation

    These are a number of definitions to make the code run on various systems 
    ( like e.g. win32... ) and some other general definitions used by the library.

    @defgroup alloc       Waveform memory allocation
    @defgroup analysis    Analysis routines
    @defgroup calib       Calibration routines
    @defgroup dsp         Digital signal processing routines
    @defgroup orbit       Beam orbit generation
    @defgroup interface   Front-end interface
    @defgroup message     Error/warning messages
    @defgroup nr          Numerical routines
    @defgroup processing  BPM signal processing
    @defgroup rf          RF simulation routines
    @defgroup sim         BPM signal simulation routines

    @mainpage
    @author  Bino Maiheu, University College London
    @author  Mark Slater, University of Cambridge
    @author  Alexey Lyapin, University College London
    @author  Stewart Boogert, Royal Holloway University of London


    @section Introduction

      This is libbpm :)


    @section Libbpm compilation

    @subsection Compilation under Linux/Unix/MacOS

       Bla bla bla

    @subsection Note on Compilation under Windows

       To compile libbpm under windows, it is best to use the MinGW + MSYS
       environment which enables one to build native libraries under windows 
       (dll). For this you need to declare some routines during the build process
       using the dllexport macro that MinGW defines. So when you want to compile
       this library as a DLL, set the BUILD_DLL define statement active below.
       Or compile using -DBUILD_DLL.  When you want to use this headerfile to for 
       linking with the bpm.ddl  library, undefine the BUILD_DLL, this will 
       enable the compiler to import routines from libbpm in other programs
       from the ddl. Under linux  it does not make a difference as the if statement
       checks first for the existence of the DLL_EXPORT and __WIN32__ macros.
    
*/

#ifndef BPMDEFS_H__
#define BPMDEFS_H__

/* -----------------------------------------------------------------------------
   includes
   -------------------------------------------------------------------------- */


/* -----------------------------------------------------------------------------
   macro definitions
   -------------------------------------------------------------------------- */

/* #define BUILD_DLL */
#if defined ( DLL_EXPORT ) && ( defined ( _WIN32 ) || defined ( __WIN32__ ) )
#  if defined BUILD_DLL
#    define EXTERN __declspec( dllexport ) 
#  else 
#    define EXTERN __declspec( dllimport )
#  endif
#else
#  define EXTERN extern
#endif

#ifndef _GNU_SOURCE
#define _GNU_SOURCE /* needed for some functions (e.g. like 'round') */
#endif

#ifndef TRUE
#  define TRUE  (1==1)
#endif 
#ifndef FALSE
#  define FALSE (0==1)
#endif 

#define SQR( a ) ( (a) * (a) )
#define ABS( a ) ( (a) < 0 ? -(a) : (a) )
#define SWAP( a, b ) tempr=(a);(a)=(b);(b)=tempr

#ifdef PI
# undef PI
#endif 
#define PI 3.14159265358979323846264338328 /* echo "scale=100;4*a(1)"|bc -l  ;) */

#define BPM_SUCCESS  0
#define BPM_FAILURE  1

#define Re 0
#define Im 1

/* -----------------------------------------------------------------------------
   typedefs, enums and other declarations
   -------------------------------------------------------------------------- */

/* -----------------------------------------------------------------------------
   function prototypes and declarations
   -------------------------------------------------------------------------- */

#endif /* #ifndef BPMDEFS_H__ */
/*@}*/
/* ================================ end of file ============================= */

