/**
   @file
   @ingroup interface
*/

#include <bpm/bpm_interface.h>
#include <bpm/bpm_messages.h>

int get_header( FILE *file, double *version, int *num_structs ){

  if ( ! file ) {
    bpm_error( "Invalid file pointer in get_header(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  if ( ! version ) {
    bpm_error( "Invalid version pointer in get_header(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  if ( ! num_structs ) {
    bpm_error( "Invalid number pointer in get_header(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  /*--------------------------------------------------------------------------
    Keep loading in lines until a non-comment is found and assume this is the header
    --------------------------------------------------------------------------
  */
  
  char line[255];
  char *tok;
  *version = 0;
  *num_structs = 0;

  while ( fgets( line, 255, file ) != NULL ) {

    // Check for comments
    tok = strtok( line, " " );
    if (tok[0] == '#') continue;

    // Assume this is the header
    *version = atof( tok );
    tok = strtok( NULL, " " );
    *num_structs = atoi( tok );
    break;

  }

  // Check for sensible values
  if ( *version <= 0 ) {
    char buf[255];

    sprintf( buf, "Strange value for the version (\"%d\") found in get_header(...)",
	     *version );
    bpm_error( buf, __FILE__, __LINE__ );

    return BPM_FAILURE;
  }

  if ( *num_structs <= 0 ) {
    char buf[255];

    sprintf( buf, "Strange value for the number of structs (\"%d\") found in get_header(...)",
	     *num_structs );
    bpm_error( buf, __FILE__, __LINE__ );

    return BPM_FAILURE;
  }
      
  return BPM_SUCCESS;
}
