/**
   @file
   @ingroup interface
*/

#include <bpm/bpm_interface.h>
#include <bpm/bpm_messages.h>

#define MAX_ARGS 200

int load_struct( FILE *file, char ***arg_list, char ***val_list, int *num_args ){

  if ( ! file ) {
    bpm_error( "Invalid file pointer in load_struct(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  if ( ! arg_list ) {
    bpm_error( "Invalid argument list pointer in load_struct(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  if ( ! val_list ) {
    bpm_error( "Invalid value list pointer in load_struct(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  if ( ! num_args ) {
    bpm_error( "Invalid pointer to number of arguments integer in load_struct(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  /*--------------------------------------------------------------------------
    Keep loading in lines until a non-comment is found and assume this is the
    start of the structure
    --------------------------------------------------------------------------
  */
  
  char line[255];
  char *tok;
  int in_struct = 0, syn_error = 0;

  char *temp_args[MAX_ARGS], *temp_vals[MAX_ARGS];
  *num_args = 0;

  while ( 1 ) {
    
    if ( fgets( line, 255, file ) == NULL ) {
      
      // End of file reached
      bpm_error( "End of file found when loading structure in load_struct(...)",
		 __FILE__, __LINE__ );
      return BPM_FAILURE;
    }

    if ( line[0] == '{' ) {

      // Check for open brace
      if (in_struct){
	syn_error = 1;
	break;
      }

      in_struct = 1;
    }
    else if ( line[0] == '}' ) {
      // End of definition
      break;
    }
    else if ( in_struct == 1 ) {
      
      // Strip whitespaces
      int strip_idx = 0;
      while ( isspace( line[strip_idx++] ) );

      // Check for blank lines
      if ((--strip_idx) == strlen(line)) continue;
      
      // Start reading in the strucutre parameters
      tok = strtok( &(line[strip_idx]), " \n\t" );
      temp_args[*num_args] = (char*) calloc( strlen(tok), sizeof(char) );
      strcpy( temp_args[*num_args], tok );
      
      // For the values of the parameters, read in the whole string in case
      // of several parameters
      tok = strtok( NULL, "#\n" );

      // strip whitespaces from front and back
      int strip_front_idx = 0;
      while ( isspace( tok[++strip_front_idx] ) );

      int strip_back_idx = strlen(tok);
      while ( isspace( tok[--strip_back_idx] ) );
      tok[strip_back_idx+1] = '\0';

      temp_vals[*num_args] = (char*) calloc( strlen(&(tok[strip_front_idx])), sizeof(char) );
      strcpy( temp_vals[*num_args], &(tok[strip_front_idx]) );
      
      ++(*num_args);
      if (*num_args == MAX_ARGS) {
	
	char buf[255];
	sprintf( buf, "Number of parameters in structure exceeded MAX_ARGS (%d) in load_struct", MAX_ARGS );
	
	bpm_error( buf, __FILE__, __LINE__ );
	return BPM_FAILURE;
      }
    }
    
  }

  // Transfer the parameters and values to the user supplied variables
  int i;
  *arg_list = (char**) calloc( *num_args, sizeof(char*) );
  *val_list = (char**) calloc( *num_args, sizeof(char*) );
  
  for ( i = 0; i < *num_args; i++ ) {

    (*arg_list)[i] = (char*) calloc( strlen(temp_args[i]), sizeof(char) );
    strcpy( (*arg_list)[i], temp_args[i] );
    
    (*val_list)[i] = (char*) calloc( strlen(temp_vals[i]), sizeof(char) );
    strcpy( (*val_list)[i], temp_vals[i] );
 
  }
  
  return BPM_SUCCESS;
}
