/**
   @file
   @ingroup interface
*/

#include <bpm/bpm_interface.h>
#include <bpm/bpm_messages.h>
#include <bpm/bpm_version.h>

int save_signals( char *fname, bpmsignal_t *sigs, int num_evts ) {

  if ( ! fname  || ! sigs ) {
    bpm_error( "Invalid pointer in save_signals(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  /*--------------------------------------------------------------------------
    Loop over the events and bpms and save the waveforms
    --------------------------------------------------------------------------
  */
  
  int ibpm, ievt, s;

  FILE *f = fopen( fname, "w+" );
  if ( !f ) {
    bpm_error( "Unable to open file for writing in save_signals(...)",
	       __FILE__, __LINE__ );
  }

  fprintf(f, "# Header (num_evts)\n%f %d\n", BPM_VERSION_VAL, num_evts);

  for (ievt = 0; ievt < num_evts; ievt++ ) {
    
    fprintf( f, "# Event %d\n%d ",  ievt, sigs[ievt].ns );
    
    for (s = 0; s < sigs[ievt].ns; s++ ) {
      
      fprintf(f, "%d ", sigs[ievt].wf[s] );
    }
    
    fprintf(f, "\n" );
    fflush(f);
  }

  fclose(f);

  return BPM_SUCCESS;
}
