/**
   @file
   @ingroup orbit
*/
#include <bpm/bpm_messages.h>
#include <bpm/bpm_orbit.h>

int generate_bpm_orbit( beamconf_t *beam, bpmconf_t *bpm ) {

  if ( ! beam ) {
    bpm_error( "Invalid pointer arguments in generate_bpm_orbit(...)",
               __FILE__, __LINE__ ); 
    return BPM_FAILURE;
  }

  if ( ! bpm ) {
    bpm_error( "Invalid pointer arguments in generate_bpm_orbit(...)",
               __FILE__, __LINE__ ); 
    return BPM_FAILURE;
  }

  // PLEASE NOTE: THIS IS A STUB OF FUNCTION TO ALLOW TESTING
  // STEWART IS GOING TO ADD THE PROPER VERSION SOON

  int i = 0;
  
  struct v3 nl;     /* direction of the line */
  struct v3 pl;     /* point on the line */
  struct v3 np;     /* normal of the plane */
  struct v3 pp;     /* point in the plane */
  struct v3 xp;     /* coordinate system x unit vector */
  struct v3 yp;     /* coordinate system y unit vector */
  struct v3 zp;     /* coordinate system z unit vector */
  struct m33 rotp;  /* rotation matrix in yaw,roll,pitch of BPM */
  double lambda;    /* parametric line / plane intersection solution */
  struct v3 vtemp;  /* temporary vector */
  struct v3 v;      /* solution */
  struct v3 lv;     /* local BPM coordinates solution */


    /* form BPM detector plane */
    pp.x = bpm->geom_pos[0]; 
    pp.y = bpm->geom_pos[1]; 
    pp.z = bpm->geom_pos[2]; 
    xp.x = 1.0; xp.y = 0;   xp.z = 0;
    yp.x = 0;   yp.y = 1.0; yp.z = 0;
    zp.x = 0;   zp.y = 0;   zp.z = 1.0;

    m_rotmat(&rotp, bpm->geom_tilt[0], bpm->geom_tilt[1], bpm->geom_tilt[2]);
    v_matmult(&rotp,&xp);
    v_matmult(&rotp,&yp);
    v_matmult(&rotp,&zp);
    
    /* m_print(&rotp); */
    
    v_copy(&np,&xp);
    v_cross(&np,&yp);
    
    /*    v_print(&np);
	  v_print(&zp); */
           
    /* find intersection of beam with BPM */
    pl.x = beam->beampos[0];
    pl.y = beam->beampos[1];
    pl.z = bpm->geom_pos[2];

    beam->beamslope[0] = 3.1415926/4;
    beam->beamslope[1] = 3.1415926/2;

    nl.x = sin(beam->beamslope[0]) * cos(beam->beamslope[1]);
    nl.y = sin(beam->beamslope[0]) * sin(beam->beamslope[1]);
    nl.z = cos(beam->beamslope[0]);
    
    /* intersection solution */    
    v_copy(&vtemp,&pp);
    v_sub(&vtemp,&pl);
    lambda = v_dot(&vtemp,&np)/v_dot(&np,&nl);
    /*    printf("generate_beam_orbit> lambda : %f \n",lambda); */
    v_copy(&v,&nl);
    v_scale(&v,lambda);
    v_add(&v,&pl);
    
    /*v_print(&v);*/

    /* beam position in coordinate system of bpm */
    v_copy(&lv,&v);
    v_sub(&lv,&pp);
    
    beam->bpmhit[0] = v_dot(&lv,&xp);
    beam->bpmhit[1] = v_dot(&lv,&yp);
    beam->bpmhit[2] = v.z;
    
    lv.x = beam->bpmhit[0];
    lv.y = beam->bpmhit[1];
    lv.z = beam->bpmhit[2];
        
    /*v_print(&lv);    */
    
    /*
    // Old and crude version - uncomment for checking 
    beam->bpmhit[0] = beam->beampos[0] - bpm->geom_pos[0];
    beam->bpmhit[1] = beam->beampos[1] - bpm->geom_pos[1];
    beam->bpmhit[2] = 0;
    */
    
    
    beam->bpmtilt[0] = beam->beamslope[0] - bpm->geom_tilt[0];
    beam->bpmtilt[1] = beam->beamslope[1] - bpm->geom_tilt[1];
    
    return BPM_SUCCESS;
}
