/**
   @file
   @ingroup rf
   @brief   libbpm rf simulation routines

   The header file for RF routines

   Need to check in how far these routines are redundant, bpmdsp can 
   replace most of the filtering routines here !
*/

/** @addtogroup rf */
/** @{ */

#ifndef BPMRF_H__
#define BPMRF_H__

/* -----------------------------------------------------------------------------
   includes
   -------------------------------------------------------------------------- */
#include <math.h>
#include <bpm/bpm_defs.h>
#include <bpm/bpm_interface.h>

/* -----------------------------------------------------------------------------
   macro definitions
   -------------------------------------------------------------------------- */

/* -----------------------------------------------------------------------------
   typedefs, enums and other declarations
   -------------------------------------------------------------------------- */
#ifdef __cplusplus
extern "C" {
#endif

/* -----------------------------------------------------------------------------
   function prototypes and declarations
   -------------------------------------------------------------------------- */
  /**
     Numer of samples in the rf waveform representations, default value 
     is 2^16 = 65536
   */
  EXTERN int      rf_nsamples; 

  /**
     Effective sampling frequency for the rf waveform representations, 
     default value is 20 GHz
  */
  EXTERN double   rf_samplefreq; 


  EXTERN int      rf_setup( int nsamples, double sfreq );

  EXTERN int      rf_rectify( double **IF );
  
  EXTERN int      rf_filter( double **RF, enum rffiltertype_t filttype, int nfiltpar, 
			     double *pars );

  /*
    Helper function that calls individual filter routines given the parameters
  */
  EXTERN int      rf_butterworthlowpass( double **RF, int order, double fc );
  /*
    Apply a low pass butterworth filter to the given waveform
  */

  EXTERN int      rf_butterworthbandpass( double **RF, int order, double f0, double BW );
  /*
    Apply a band pass butterworth filter to the given waveform
  */

  EXTERN int      rf_butterworthhighpass( double **RF, int order, double fc );
  /*
    Apply a high pass butterworth filter to the given waveform
  */
  EXTERN int      rf_complexFFT( double **in, double **out, int dir );
  /*
    Perform a complex fft of the given waveform. if dir == NR_FFTFORWARD, forward
    fft is calculated, if dir == NR_FFTBACKWARD, the inverse is calcuated
  */

  EXTERN int      rf_addLO( double amp, double lofreq, enum bpmphase_t type,
			    double phi0, double d_phi, double **LO );
  /*
    Generates an LO waveform with an amplitude amp, frequency
    lofreq, constant phase phi0 if d_phi = 0, constant phase phi0
    and phase jitter d_phi if d_phi<>0 and bpmphase_t = locked,
    random phase between phi0-d_phi and phi0+d_phi otherwise
  */
  EXTERN int rf_mixer( double *RF_Re, double *RF_Im, double **LO, double *IF );

  /*
    ideal mixer
  */
  EXTERN int rf_amplify( double *RF, double dB );
  /*
    need to put in the proper compression point stuff etc...
  */


#ifdef __cplusplus
}
#endif

#endif /* #ifndef BPMRF_H__ */
/** @} */
/* ================================ end of file ============================= */
