/**
   @file
   @ingroup rf
*/
#include <bpm/bpm_interface.h>
#include <bpm/bpm_rf.h>
#include <bpm/bpm_nr.h>
#include <math.h>

/**
   Adds an LO signal to the waveform
   
   @returns BPM_SUCCESS upon success, BPM_FAILURE upon failure
*/
int rf_addLO( double amp, double lofreq, enum bpmphase_t type, double phi0, double d_phi,
	      double **LO ) {

  if ( ! LO ) {
    bpm_error( "Invalid LO pointer in rf_addLO(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  int i;
  double t, phase;

  // Phase type
  
  if ( type == locked ) {

    if ( d_phi == 0 ) phase= phi0; else phase= nr_rangauss(phi0, d_phi);

  } else {

    if ( d_phi == 0 ) phase= phi0; else nr_ranuniform( phi0-d_phi, phi0+d_phi );

  }

//double phase = ( type == locked ? 0 : 0 ); //nr_rangauss(phi0, d_phi) : nr_ranuniform(phi0-d_phi, phi0+d_phi) );

  for( i = 0; i < rf_nsamples; i++ ) {

    // calculate the sampling time
    t = i / rf_samplefreq;

    LO[i][Re] +=  amp * cos( 2.* PI * lofreq * t + phase );
    LO[i][Im] +=  amp * sin( 2.* PI * lofreq * t + phase );

  } /* for( i = 0; i<nS; i++ ) */

  return BPM_SUCCESS;
}
// end of file
