/**
   @file
   @ingroup rf
*/
#include <bpm/bpm_interface.h>
#include <bpm/bpm_rf.h>

/**
   Amplifies the signal by the level dB. The voltage gain is calculated:
   
   gain = sqrt( 10^(db/20) )
   
   @returns BPM_SUCCESS upon success, BPM_FAILURE upon failure
*/
int rf_amplify( double *RF, double dB ) {

  int i;
 
  if ( ! RF ) {
    bpm_error( "Invalid pointer arguments in rf_amplify(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  double volt_gain = sqrt( pow ( 10., dB / 20. ) );

  // Loop over the RF signal
  for ( i = 0; i < rf_nsamples; i++ ) {

    RF[i] *= volt_gain;
    //RF[i][Im] *= volt_gain;
  }

  return BPM_SUCCESS;
}
// end of file
