/**
   @file
   @ingroup rf
*/
#include <bpm/bpm_interface.h>
#include <bpm/bpm_rf.h>
#include <bpm/bpm_alloc.h>
#include <bpm/bpm_nr.h>

/**
   Apply a band pass Butterworth filter which is effectively a combination of a low
   and a high pass filters.
   
   @param RF       the waveform
   @param order    The order of the filter
   @param f0       Central filter frequency in MHz
   @param BW       Bandwidth in MHz
   
   @returns BPM_SUCCESS upon success, BPM_FAILURE upon failure
*/
int rf_butterworthbandpass( double **RF, int order, double f0, double BW ) {


  int i = 0;
  double f;

  if ( ! RF ) {
    bpm_error( "Invalid RF pointer in rf_butterworthlowpass(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  if ( f0 <= 0 ) {
    bpm_error( "Central frequency <= 0 in rf_butterworthlowpass(...)",
	       __FILE__, __LINE__);
    return BPM_FAILURE;
  }

  if ( BW <= 0 ) {
    bpm_error( "Bandwidth <= 0 in rf_butterworthlowpass(...)",
	       __FILE__, __LINE__);
    return BPM_FAILURE;
  }

  if ( order <= 0 ) {
    bpm_error( "Order of filter =< 0 in rf_butterworthlowpass(...)",
	       __FILE__, __LINE__);
    return BPM_FAILURE;
  }

  double fc1 = f0 - .5 * BW;
  double fc2 = f0 + .5 * BW;

  rf_butterworthhighpass( RF, order, fc1 );
  rf_butterworthlowpass( RF, order, fc2 );

  return BPM_SUCCESS;
}
// end of file
