/**
   @file
   @ingroup rf
*/
#include <bpm/bpm_interface.h>
#include <bpm/bpm_rf.h>
#include <bpm/bpm_alloc.h>
#include <bpm/bpm_nr.h>

/**
   Perform a complex FFT on the input waveform
   
   @param in       the input waveform
   @param out      the output FFT
   @param dir      forward or inverse?
   
   @returns BPM_SUCCESS upon success, BPM_FAILURE upon failure
*/
int rf_complexFFT( double **in, double **out, int dir ) {


  int i = 0;
  double f;

  if ( ( ! in ) || ( ! out ) ) {
    bpm_error( "Invalid pointers in rf_complexFFT(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  // Pack the temp array with zeros to make a power of two
  int ns = 1;
  while ( ns < rf_nsamples ) { 
    ns *= 2;
  }


  // Allocate new array for nr_four1
  double *cp = (double*) calloc( 2*ns+1, sizeof(double) );

  if( ! cp ) {
    bpm_error( "Could not allocate memory in rf_complexFFT(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }
  
  // Transfer waveform
  for( i = 0; i < rf_nsamples; i++ ) {

    // translate to what nr_four1 needs
    cp[2*i+1] = in[i][Re];  // real part
    cp[2*i+2] = in[i][Im]; // imaginary part

  }

  if ( nr_four1( cp, ns, dir ) == BPM_FAILURE ) {
    free(cp);
    bpm_error( "Unable to execute nr_four routine in rf_complexFFT(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  // now store in fft...
  for( i = 0; i < rf_nsamples; i++ ) {
    out[i][Re] = cp[2*i+1];
    out[i][Im] = cp[2*i+2];
  }

  free (cp);

  return BPM_SUCCESS;
}
// end of file
