/**
   @file
   @ingroup rf
*/
#include <bpm/bpm_interface.h>
#include <bpm/bpm_rf.h>

/**
   Applies the filter to the RF waveform
   
   @param RF       the waveform
   @param filttype the filter type
   @param nfiltpar the number of filter parameters
   @param pars     the array of filter parameters
   
   @returns BPM_SUCCESS upon success, BPM_FAILURE upon failure
*/
int rf_filter( double **RF, enum rffiltertype_t filttype, int nfiltpar, double *pars ) {


  if ( ! RF ) {
    bpm_error( "Invalid RF pointer in rf_filter(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  // Call the approriate filter function
  switch (filttype) {
    
  case nofilter:
    // No filter selected
    break;

  case butterworth_low_pass:
    // Low pass butterworth
    if ( nfiltpar != 2 ) {
      bpm_error( "Incorrect number of parameters given for a butterworth low pass filter",
		 __FILE__, __LINE__ );
    }

    rf_butterworthlowpass( RF, (int)pars[0], pars[1] );
    break;

  case butterworth_band_pass:
    // band pass butterworth
        if ( nfiltpar != 3 ) {
      bpm_error( "Incorrect number of parameters given for a butterworth low pass filter",
		 __FILE__, __LINE__ );
    }

    rf_butterworthbandpass( RF, (int)pars[0], pars[1], pars[2] );
    break;

  case butterworth_high_pass:
    // high pass butterworth
        if ( nfiltpar != 2 ) {
      bpm_error( "Incorrect number of parameters given for a butterworth high pass filter",
		 __FILE__, __LINE__ );
    }

    rf_butterworthhighpass( RF, (int)pars[0], pars[1] );
    break;
  }

  return BPM_SUCCESS;
}
// end of file
