/**
   @file
   @ingroup sim
*/

#include <bpm/bpm_messages.h>
#include <bpm/bpm_simulation.h>
#include <bpm/bpm_rf.h>
#include <bpm/bpm_nr.h>

/**
   Add the given amount of amplitude noise to the array
   
   @param amplnoise The amplitude noise to add to the waveform (in Volts)
   @param IF_Re The real part of the waveform
   @param IF_Im The imaginary part of the waveform
   
   @return BPM_SUCCESS upon succes, BPM_FAILURE upon failure
*/
int add_amplnoise( double amplnoise, double *IF_Re, double *IF_Im ) {


  int i;
  double mag, phase;

  if ( ! IF_Re || ! IF_Im ) {
    bpm_error( "Invalid pointer arguments in add_amplnoise(...)", 
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  for ( i = 0; i < rf_nsamples; i++ ) {

    mag = nr_rangauss( 0, amplnoise );
    phase = nr_ranuniform( 0, 2 * PI );

    IF_Re[i] += mag * cos (phase);
    IF_Im[i] += mag * sin (phase);

  }

  return BPM_SUCCESS;
}

// end of file
