/**
   @file
   @ingroup sim
*/

#include <bpm/bpm_messages.h>
#include <bpm/bpm_simulation.h>
#include <bpm/bpm_rf.h>

/**
   Builds up an RF waveform from the given amp, phase frequency and tdecay...
   uses the rf_ parameters which dictate the internal representation of the
   RF wave
*/
int add_wave( double amp, double phase, double freq, double ttrig, 
	      double tdecay, double **RF ) {

  double curr_time;
  int i;

  if ( ! RF ) {
    bpm_error( "Invalid pointer argument in add_wave(...)", 
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  for ( i = 0; i < rf_nsamples; i++ ) {

    curr_time = i / rf_samplefreq;
    
    if ( curr_time >= ttrig ) {
      RF[i][Re] += amp * exp( - ( curr_time - ttrig ) / tdecay ) *
	cos( 2.* PI * freq * ( curr_time - ttrig ) + phase );
      RF[i][Im] += amp * exp( - ( curr_time - ttrig ) / tdecay ) *
	sin( 2.* PI * freq * ( curr_time - ttrig ) + phase );
    }
  }
  
  return BPM_SUCCESS;
}

// end of file
