/**
   @file
   @ingroup sim
*/
#include <bpm/bpm_messages.h>
#include <bpm/bpm_simulation.h>
#include <bpm/bpm_rf.h>
#include <bpm/bpm_alloc.h>

/**
   Generate monopole waveform
*/
int generate_monopole( bpmconf_t *bpm, beamconf_t *beam, bpmsignal_t *sig ) {


  if ( ! bpm || ! beam || ! sig ) {
    bpm_error( "Invalid pointer arguments in generate_monopole(...)",
	       __FILE__, __LINE__ );
    return BPM_FAILURE;
  }

  // Compute the monopole response of the BPM
  double amp, phase;
  get_monopole_response( beam->charge, bpm->cav_chargesens, beam->arrival_time, bpm->cav_freq,
			 &amp, &phase );

  // Generate the waveoform
  double **RF = alloc_complex_wave_double( rf_nsamples );
  add_wave( amp, phase, bpm->cav_freq, beam->arrival_time + bpm->digi_trigtimeoffset, bpm->cav_decaytime, RF );

  // Mix down
  double **LO = alloc_simple_wave_double( rf_nsamples );
  double **IF = alloc_complex_wave_double( rf_nsamples );

  // Arbitrarily set the LO amplitude to 10dBm = 0.01 W == sqrt(0.01 * 50) V
  double lo_amp = sqrt( 0.5 ); 
  
  //rf_addLO( lo_amp, bpm->rf_LOfreq, bpm->cav_phasetype, 0, 0, LO );
  //rf_mixer( RF, LO, IF );
  
  // Filter the signal
  //rf_filter( IF, bpm->rf_filtertype, bpm->rf_nfiltpars, bpm->rf_filterpars );

  // Amplify
  //rf_amplify( IF, bpm->rf_gain );
  
  // Add some noise - currently assume ADC goes from -1V -> +1V
  double conv = (double) pow( 2, bpm->digi_nbits ) / 2.;
  double noise = bpm->digi_ampnoise / conv;
  //add_amplnoise( noise, IF );
  
  // digitise the waveform - currently assume ADC goes from -1V -> +1V
  int *wf = alloc_simple_wave_int( bpm->digi_nsamples );
  //digitise( IF, bpm->digi_nbits, bpm->digi_voltageoffset, bpm->digi_freq, conv, 
//	    bpm->digi_nsamples, wf);
  
  // Finally, store the waveform in the signal structure
  sig->wf = wf;
  sig->ns = bpm->digi_nsamples;
  
  // Clean up
  free_complex_wave_double( RF, rf_nsamples );
  free_complex_wave_double( IF, rf_nsamples );
  free_simple_wave_double( LO );

  return BPM_SUCCESS;
}

// end of file
