/**
   @file
   @ingroup sim
*/
#include <bpm/bpm_messages.h>
#include <bpm/bpm_simulation.h>
#include <math.h>

/**
   Calculate the response of a dipole signal given an incoming bunch
   
   @param bunchcharge The charge of the bunch (in nC)
   @param bunchlength The length of the bunch (in mm)
   @param pos The position of the beam (in mm)
   @param possens The position sensitivty of the BPM (in V/nC/mm)
   @param slope The slope of the beam (in rad)
   @param slopesens The slope sensitivity of the BPM (in V/nC/urad)
   @param tilt The tilt of the bunch (in urad)
   @param tiltsens The tilt sensitivty of the BPM (V/nC/urad)
   @param amp the amplitude of the waveform at the arrival time
   @param phase the phase of the waveform at the arrival time
*/
int get_dipole_amp( double bunchcharge, double bunchlength,
                    double pos, double possens,
                    double slope, double slopesens,
                    double tilt, double tiltsens,
                    double *amp, double *phase ){


  double I =        ( pos * possens ) * bunchcharge;
  double Q = (-1) * ( slope * slopesens ) * bunchcharge;
  //need to add the tilt

  *amp = sqrt( I * I + Q * Q );
  if ( *amp == 0 ) *phase = 0; else
  *phase = acos(I / *amp); // ( I > 0 ? asin(I / *amp) : PI - asin(I / *amp) );
  
  return BPM_SUCCESS;
}

// end of file
