/**
   @file
   @ingroup sim
*/
#include <bpm/bpm_messages.h>
#include <bpm/bpm_simulation.h>

/**
   Calculate the response of a dipole signal given an incoming bunch
   
   NOTE: Still have phase questions!
   **Alex**May need to include the bunch length**
   
   @param bunchcharge The charge of the bunch (in nC)
   @param chargesens The charge sensitivty of the BPM **Alex**remove this**
   @param pos The position of the beam (in um)
   @param possens The position sensitivty of the BPM **Alex**mV/nC/mm**
   @param tilt The tilt of the beam (in urad)
   @param tiltsens The tilt sensitivty of the BPM **Alex**mV/nC/mrad**
   @param arrivaltime The beam arrival time
   @param cavityfreq The frequency of the cavity
   @param amp the amplitude of the waveform at the arrival time
   @param phase the phase of the waveform at the arrival time
*/
int get_dipole_response( double bunchcharge, double chargesens,
			 double pos, double possens, 
			 double tilt, double tiltsens,
			 double arrivaltime, double cavityfreq,
			 double *amp, double *phase ) {

  // The pos/tilt sensistivity is given at 1.e10 electrons so calculate
  // as normal and scale with appropriate charge
  double I = ( pos * possens ) * bunchcharge;
  double Q = ( tilt * tiltsens ) * bunchcharge;

  *amp = sqrt( I * I + Q * Q );
  *phase = ( I > 0 ? asin(I / *amp) : PI - asin(I / *amp) );
    
  return BPM_SUCCESS;
}

// end of file
