/**
   @file
   @ingroup sim
*/
#include <bpm/bpm_messages.h>
#include <bpm/bpm_simulation.h>
#include <math.h>

/**
   Calculate the response of a dipole signal given an incoming bunch
   
   @param bunchcharge The charge of the bunch (in nC)
   @param bunchlength The length of the bunch (in mm)
   @param chargesens The charge sensitivty of the BPM (V/nC)
   @param amp the amplitude of the waveform at the arrival time
   @param phase the phase of the waveform at the arrival time
*/
int get_monopole_amp( double bunchcharge, double bunchlength,
                    double chargesens,
                    double *amp, double *phase ){


  double I = 0;
  double Q = (-1) * chargesens * bunchcharge;
  //need check the phase

  *amp = sqrt( I * I + Q * Q );
  if ( *amp == 0 ) *phase = 0; else
  *phase = acos(I / *amp); // ( I > 0 ? asin(I / *amp) : PI - asin(I / *amp) );
  
  return BPM_SUCCESS;
}

// end of file
